// ------------------------------------------------------------//
// DEMO 7 : Ad Pacer trigger with AD POLLING with calibation   //
// Compiler: Turbo   C++ 3.0, Mode Large                       //
// ------------------------------------------------------------//

#include "PIO.H"

static int wSysConfig=0;
float fAdData[1000];
WORD  AdValue[1000];
WORD wBase,wIrq;
WORD wDELAY=23,time=0;
WORD EepAddr,EepData,wZERO;


//---------------------------------------------------------
WORD Pio821_Delay(WORD wDownCount)
{
  WORD h,l;
  int count;
  wDownCount &= 0xff;
  if (wDownCount<1) wDownCount=1;

 // Clock in=2M --> count 4000 = 1 ms ,count 1 = 0.5 us
  l=wDownCount&0xff;
  wDownCount=wDownCount / 256;
  h=wDownCount&0xff;

  outportb(wBase+0xcc,0xb4);	             // CW of 8254,mode_0, counter_2
  outportb(wBase+0xc8,l);	             // counter_2 low byte first
  outportb(wBase+0xc8,h);	             // counter_2 high byte ,0x07D0=2000

  outportb(wBase+0xcc,0x80);	             // latch counter_2
  l=inportb(wBase+0xc8);	             // delay starting two clks
  h=inportb(wBase+0xc8);

  for (count=32767;count>0;count--)
  {
	  outportb(wBase+0xcc,0x80);         // latch counter_2
	  l=inportb(wBase+0xc8);
	  h=inportb(wBase+0xc8);
	  if (h>=0x80) return NoError;
  }
  return TimeOut;
}

//---------------------------------------------------
void SetupTimer(WORD wCoef)
{
   WORD   cmd,lsb,msb;
   outportb(wBase+0xcc, 0xb4);
   outportb(wBase+0xc8, wCoef&0xff);
   outportb(wBase+0xc8, wCoef>>8);
}

//-------------------------------------------------------
void Pio821_AdsPacer(float fAdVal[],WORD wNum, WORD wSamplingDiv)
{
   WORD    wVal,i,j,k;
   WORD    MSB,LSB,wTime,sts;
   float   fMAX;

   switch (wSysConfig)
   {
       case 0 :  fMAX=(float)5.0  ;   break;
       case 1 :  fMAX=(float)2.5  ;   break;
       case 2 :  fMAX=(float)1.25 ;   break;
       case 3 :  fMAX=(float)0.625;   break;
   }

   outportb(wBase+0xe4,0xf2);      // Set trigger mode=Pacer trigger
   SetupTimer(wSamplingDiv);       // sampling rate=2M/wSamplingDiv

   EepAddr=wBase+0xec;             //set the EEPROM address
   EEP_READ(2,&j,&k);
   wZERO=(j<<8)+k;                 //read the calibation value to wZERO

   for(i=0;i<wNum;i++)
   {

	do{
		sts=inportb(wBase+0xec);
	}while((sts&0x01)==0);

      MSB=inportb(wBase+0xd4);
      LSB=inportb(wBase+0xd0);
      
   fAdVal[i]=((float)((((MSB<<8)+LSB)&0xfff)-wZERO)/2048.0)*fMAX;
  }

  outportb(wBase+0xe4,0xf0);        		  // close Mode
 
}

//-------------------------------------------------------
WORD  Pio821_SetChannelConfig(WORD wAdChannel,WORD wConfig)
{

	WORD wDELAY;
	outportb(wBase+0x10,(UCHAR)wAdChannel);
	outportb(wBase+0x14,(UCHAR)wConfig);
	wSysConfig=wConfig;

	switch (wSysConfig)
	{
	 case 0 : wDELAY=23;      break;
	 case 1 : wDELAY=23;      break;
	 case 2 : wDELAY=23;      break;
	 case 3 : wDELAY=23;      break;

	 case 0x10 : wDELAY=23;   break;
	 case 0x11 : wDELAY=28;   break;
	 case 0x12 : wDELAY=140;  break;
	 case 0x13 : wDELAY=1300; break;
      }

	outportb(wBase+0xe0,wAdChannel);
	Pio821_Delay(wDELAY *4);
	return NoError;
}



int main(void)
{
int  i,j,k,k2,l1,l2,jj,dd,j1,i1,j2,i2;
//int  t=0,quit=0;

WORD wBoards,wRetVal,t1,t2,t3,t4,t5,High;
WORD wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice;
char c,DO,ReadBack;
FILE *stream;

clrscr();
// Step 1: Find address-mapping of PIO/PISO cards
wRetVal=PIO_DriverInit(&wBoards,0x40ff,0x03,0x10);

printf("\nThere are %d PIO-821 Cards in this PC",wBoards);
if (wBoards==0) exit(0);

printf("\n--------------- The Configuration Space ---------------");

for(i=0; i<wBoards; i++)
   {
   PIO_GetConfigAddressSpace(i,&wBase,&wIrq,&wSubVendor,&wSubDevice,
   &wSubAux,&wSlotBus,&wSlotDevice);
   printf("\nCard_%d: wBase=%x,wIrq=%x,subID=[%x,%x,%x],SlotID=[%x,%x]",i,wBase,wIrq,wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice);
   printf(" --> ");
   ShowPioPiso(wSubVendor,wSubDevice,wSubAux);
   }
      
   
PIO_GetConfigAddressSpace(0,&wBase,&wIrq,&t1,&t2,&t3,&t4,&t5);

printf("\n\nAd Pacer trigger with AD POLLING with Calibation");
printf("\nPress any key to start");
getch();


// Step 2: Enable all D/I/O port
outportb(wBase,0X31);	        	     // enable D/I/O
outportb(wBase+0xe0,0);                      // Set channel=0,Gain=0

Pio821_AdsPacer(fAdData,1000,40);          // Sample Rate= 2Mhz/200 = 10k Max

stream=fopen("data.dat","wb+");
for(i=0;i<1000;i++)
   {
    fprintf(stream,"%2.5f\n",fAdData[i]);
   }

PIO_DriverClose();
clrscr();
printf("Data already save to data.dat");

return 0;
}
/* D0 = CS of EEP  */
/* D1 = SK of EEP  */
/* D2 = DI of EEP  */
/* D0 = DO of EEP  */

COMM()	/* send start 2-bit (0,1)  */
{
EepData=4; outportb(EepAddr,EepData); /* CS=L, SK=L, DI=H */
EepData=5; outportb(EepAddr,EepData); /* CS=H, SK=L, DI=H */
EepData=7; outportb(EepAddr,EepData); /* CS=H, SK=H, DI=H */
EepData=5; outportb(EepAddr,EepData); /* CS=H, SK=L, DI=H */
}

WR_BYTE(int val)  /* send the next 8-bit */
{
int i,j;

for (i=0; i<8; i++)
    {
    EepData &= 5; outportb(EepAddr,EepData);	/* SK=L */

    j=val&0x80; val=val<<1;
    if (j==0)					/* DI=0 */
       {
       EepData &= 3; outportb(EepAddr,EepData); /* DI=L */
       }
    else					/* DI=1 */
       {
       EepData |= 4; outportb(EepAddr,EepData); /* DI=H */
       }

    EepData |= 2; outportb(EepAddr,EepData);	/* SK=H */
    }
}

RD_BYTE()   // read the next 8-bit
{
int i,j,val,k;

val=0x80; k=0;
for (i=0; i<8; i++)
    {
    EepData &= 5; outportb(EepAddr,EepData); /* SK=L */
    j=(inportb(EepAddr))&8;
    if (j!=0) k+=val;			     /* DO=1 */
    EepData |= 2; outportb(EepAddr,EepData); /* SK=H */
    val=val>>1;
    }
return(k);
}

/* ---------------------------------------------------------------*/

EEP_WR_EN()   /* write Enable */
{
COMM();
WR_BYTE(0x30);				 /* write enable command */
EepData &= 5; outportb(EepAddr,EepData); /* SK=L                 */
EepData &= 6; outportb(EepAddr,EepData); /* CS=L                 */
EepData &= 3; outportb(EepAddr,EepData); /* DI=L                 */
}

/* ---------------------------------------------------------------*/

EEP_WR_DIS()  /* write Disable */
{
COMM();
WR_BYTE(0);				 /* write enable command   */
EepData &= 5; outportb(EepAddr,EepData); /* SK=L		   */
EepData &= 6; outportb(EepAddr,EepData); /* CS=L		   */
EepData &= 3; outportb(EepAddr,EepData); /* DI=L		   */
}

/* --------------------------------------------------------------- */

EEP_WRITE(int ADDR, int HI, int LO)  /* write 16-bit data to EEP   */
{
int i,j,k;

COMM();
ADDR=ADDR|0x40; WR_BYTE(ADDR);		/* write command & address */
WR_BYTE(HI);				/* high 8-bit data	   */
WR_BYTE(LO);				/* low	8-bit data	   */

EepData &= 5; outportb(EepAddr,EepData);/* SK=L 		   */
EepData &= 6; outportb(EepAddr,EepData);/* CS=L 		   */
EepData &= 3; outportb(EepAddr,EepData);/* DI=L 		   */

for (i=0; i<10; i++)			/* delay for EEP	   */
    {
    outportb(EepAddr,EepData);
    }

EepData |= 1; outportb(EepAddr,EepData);/* CS=H 		   */

for (k=0;k<10;k++)
for (i=0;i<32767;i++)
    {
    EepData |= 2; outportb(EepAddr,EepData); /* SK=H */
    j=(inportb(EepAddr))&8;
    EepData &= 5; outportb(EepAddr,EepData); /* SK=L */

    if (j!=0)
       {
       EepData &= 6; outportb(EepAddr,EepData); /* CS=L */
       return(0);
       }
    }

err_label:
EepData=0; outportb(EepAddr,EepData); /* CS=L, SK=L, DI=L */
return(1);
}

/* --------------------------------------------------------------- */

EEP_READ(int ADDR, int *HI, int *LO) /* read 16-bit data from EEP  */
{
int i,j,k;

COMM();
ADDR=ADDR|0x80; WR_BYTE(ADDR);		/* read command & address  */

EepData &= 3; outportb(EepAddr,EepData);/* DI=L 		   */
EepData &= 5; outportb(EepAddr,EepData);/* SK=L 		   */
EepData |= 2; outportb(EepAddr,EepData);/* SK=H 		   */
EepData &= 5; outportb(EepAddr,EepData);/* SK=L 		   */

(*HI)=RD_BYTE();			/* read high 8-bit data    */
(*LO)=RD_BYTE();			/* read low  8-bit data    */

EepData &= 5; outportb(EepAddr,EepData);/* SK=L 		   */
EepData &= 6; outportb(EepAddr,EepData);/* CS=L 		   */
}
